{% extends 'farm/base.html' %}
{% load humanize typogrify %}

{% block content %}

<h2>Animals</h2>

{% for a in animal_list %}
<dl>
    <dt>{{a.title}}</dt>
    <dd>{{a.breed}}</dt>
    <dd>{{a.sex.get_display_name}}</dd>
    {% for n in a.notes.all %}
    <dd><strong>{{n.date|naturalday}}</strong>{{n.rendered_content|typogrify}}</dd>
    {% endfor %}
</dl>
{% endfor %}

{% endblock %}
